# coding: utf-8

from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.db.models.signals import post_save

from petits_cours_models import *

OCCUPATION_CHOICES = (
    ('exterieur', _(u"Extérieur")),
    ('1A', _(u"1A")),
    ('2A', _(u"2A")),
    ('3A', _(u"3A")),
    ('4A', _(u"4A")),
    ('archicube', _(u"Archicube")),
    ('doctorant', _(u"Doctorant")),
    ('CST', _(u"CST")),
)

TYPE_COTIZ_CHOICES = (
    ('etudiant', _(u"Étudiant")),
    ('normalien', _(u"Normalien")),
    ('exterieur', _(u"Extérieur")),
)

def choices_length (choices):
    return reduce (lambda m, choice: max (m, len (choice[0])), choices, 0)

class CofProfile(models.Model):
    user = models.OneToOneField(User, related_name = "profile")
    login_clipper = models.CharField("Login clipper", max_length = 8, blank = True)
    is_cof = models.BooleanField("Membre du COF", default = False)
    num = models.IntegerField ("Numéro d'adhérent", blank = True, default = 0)
    phone = models.CharField("Téléphone", max_length = 20, blank = True)
    occupation = models.CharField (_(u"Occupation"),
                             default = "1A",
                             choices = OCCUPATION_CHOICES,
                             max_length = choices_length (OCCUPATION_CHOICES))
    departement = models.CharField (_(u"Département"), max_length = 50, blank = True)
    type_cotiz = models.CharField (_(u"Type de cotisation"),
                             default = "normalien",
                             choices = TYPE_COTIZ_CHOICES,
                             max_length = choices_length (TYPE_COTIZ_CHOICES))
    mailing_cof = models.BooleanField("Recevoir les mails COF", default = False)
    mailing_bda = models.BooleanField("Recevoir les mails BdA", default = False)
    mailing_bda_revente = models.BooleanField("Recevoir les mails de revente de places BdA", default = False)
    is_buro = models.BooleanField("Membre du Burô", default = False)
    petits_cours_accept = models.BooleanField("Recevoir des petits cours", default = False)
    petits_cours_sent = models.IntegerField("Nombre de propositions petits cours reçues", default = 0)

    class Meta:
        verbose_name = "Profil COF"
        verbose_name_plural = "Profils COF"

    def __unicode__(self):
        return unicode(self.user.username)

def create_user_profile(sender, instance, created, **kwargs):
    if created:
        CofProfile.objects.get_or_create(user = instance)
post_save.connect(create_user_profile, sender = User)

class Event(models.Model):
    title = models.CharField("Titre", max_length = 200)
    location = models.CharField("Lieu", max_length = 200)
    start_date = models.DateField("Date de début", blank = True, null = True)
    end_date = models.DateField("Date de fin", blank = True, null = True)
    description = models.TextField("Description", blank = True)
    registration_open = models.BooleanField("Inscriptions ouvertes", default = True)
    old = models.BooleanField("Archiver (événement fini)", default = False)

    class Meta:
        verbose_name = "Événement"

    def __unicode__(self):
        return unicode(self.title)

class EventOption(models.Model):
    event = models.ForeignKey(Event, related_name = "options")
    name = models.CharField("Option", max_length = 200)
    multi_choices = models.BooleanField("Choix multiples", default = False)

    class Meta:
        verbose_name = "Option"

    def __unicode__(self):
        return unicode(self.name)

class EventOptionChoice(models.Model):
    event_option = models.ForeignKey(EventOption, related_name = "choices")
    value = models.CharField("Valeur", max_length = 200)

    class Meta:
        verbose_name = "Choix"

    def __unicode__(self):
        return unicode(self.value)

class EventRegistration(models.Model):
    user = models.ForeignKey(User)
    event = models.ForeignKey(Event)
    options = models.ManyToManyField(EventOptionChoice)
    paid = models.BooleanField("A payé", default = False)

    class Meta:
        verbose_name = "Inscription"
        unique_together = ("user", "event")

    def __unicode__(self):
        return u"Inscription de %s à %s" % (unicode(self.user), unicode(self.event.title))

class Survey(models.Model):
    title = models.CharField("Titre", max_length = 200)
    details = models.TextField("Détails", blank = True)
    survey_open = models.BooleanField("Sondage ouvert", default = True)
    old = models.BooleanField("Archiver (sondage fini)", default = False)

    class Meta:
        verbose_name = "Sondage"

    def __unicode__(self):
        return unicode(self.title)

class SurveyQuestion(models.Model):
    survey = models.ForeignKey(Survey, related_name = "questions")
    question = models.CharField("Question", max_length = 200)
    multi_answers = models.BooleanField("Choix multiples", default = False)

    class Meta:
        verbose_name = "Question"

    def __unicode__(self):
        return unicode(self.question)

class SurveyQuestionAnswer(models.Model):
    survey_question = models.ForeignKey(SurveyQuestion, related_name = "answers")
    answer = models.CharField("Réponse", max_length = 200)

    class Meta:
        verbose_name = "Réponse"

    def __unicode__(self):
        return unicode(self.answer)

class SurveyAnswer(models.Model):
    user = models.ForeignKey(User)
    survey = models.ForeignKey(Survey)
    answers = models.ManyToManyField(SurveyQuestionAnswer, related_name = "selected_by")

    class Meta:
        verbose_name = "Réponses"
        unique_together = ("user", "survey")

class Clipper(models.Model):
    username = models.CharField("Identifiant", max_length = 20)
    fullname = models.CharField("Nom complet", max_length = 200)

"""
class Voter(User):
    class Meta:
        proxy = True
        verbose_name = "voteur"
    def __unicode__(self):
        return u"%s %s" % (self.first_name, self.last_name)

import eav
eav.register(CofProfile)

class ManagerOnlyEavConfig(eav.registry.EavConfig):
    manager_only = True

eav.register(User, ManagerOnlyEavConfig)
"""
